/*******************************************************************************

        @file TextFormat.d
        
        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, March 2004      
        @author         Kris


*******************************************************************************/

module mango.io.TextFormat;

private import mango.format.Format;

/*******************************************************************************

        Constructs printf-style output. This is an adapter for "old" printf 
        formatting, and writes it's output into a lookaside buffer. Content
        from the buffer can easily be passed to an IWriter instance.

        @code
        // create a TextFormat instance
        TextFormat format = new TextFormat (256);

        // write text to Stdout
        Stdout (format ("%d green bottles, sitting on a wall\n", 10));

        // or access the content, post formatting ...
        char[] text = format.content;
        @endcode

        This can be really handy when you wish to format text in conjunction 
        with a Logger. Please note that the class is stateful, and therefore 
        is not shareable across multiple threads. 
        
*******************************************************************************/

class TextFormat
{
        alias format opCall;
              
        private int             count;
        private char[]          output;
        private Format.Sprintf  sprintf;

        /***********************************************************************
        
                Set the maximum buffer length

        ***********************************************************************/

        this (int limit, char[] workspace = null, Format.DblFormat df = null)
        {
                output = new char [limit]; 
                sprintf (output, workspace, df);
        }

        /***********************************************************************
                
                format a set of arguments

        ***********************************************************************/

        char[] format (char[] fmt, ...) 
        { 
                count = sprintf.format (fmt, _arguments, _argptr);
                return content;
        }

        /***********************************************************************
        
                Address formatted output as a char[]

        ***********************************************************************/

        char[] content ()
        {
                return output [0..count];
        }
}
