/*******************************************************************************

        @file IResource.d
        
        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, March 2004      
        @author         Kris


*******************************************************************************/

module mango.io.model.IResource;

/*******************************************************************************

        Protocol for acquiring and releasing a resource such as a file, or
        socket. This isn't strictly necessary, but will aid migration to a
        multi-threaded runtime environment.

*******************************************************************************/

interface IResource 
{
        /***********************************************************************
        
                Subclasses must implement closure mechanics. This is 
                invoked when the number of close() invocations match
                that of aquire()  --  the latter is typically called 
                immediately following the successful construction of
                an IResource implementation; one should expect close
                to be required at least once.

        ***********************************************************************/

        abstract protected void closure (); 
           
        /***********************************************************************
        
                Return the lock used by this resource

        ***********************************************************************/

        Object getLock ();

        /***********************************************************************
        
                Set an alternate lock. Typically one would use this to 
                synchronize multiple different Resources via a common
                object. 

        ***********************************************************************/

        void setLock (Object lock);

        /***********************************************************************
        
                hold a reference to this resource. The caller is now
                responsible for invoking close() upon this resource;

        ***********************************************************************/

        void acquire ();

        /***********************************************************************
        
                If this resource is still open, acquire it and return true.
                This is a threadsafe test-and-set mechanism that fails only
                if the resource is already closed, or was never opened.

                Note that you are responsible for invoking close() whenever
                this method returns true.

        ***********************************************************************/

        bool acquireIfOpen ();

        /***********************************************************************
        
                release a reference to this resource

        ***********************************************************************/

        void close ();
}

