/*
 * Examples for Mango's Containers
 */

import mango.containers.Container;
import mango.containers.Iterator;
import mango.containers.ImmutableList;
import mango.containers.List;
import mango.containers.ListMutators;
import mango.containers.LinkedList;
import mango.containers.Map;
import mango.containers.ArrayList;
import mango.containers.Set;
import mango.containers.HashMap;
import mango.containers.HashSet;
import mango.containers.ConcurrentHashMap;
import mango.containers.Map;
import mango.containers.Util;

import mango.io.Stdout;

void main() {
  ImmutableList1();
  //HashMap1();
  //HashSet1();
  //ConcurrentHashMap1();
  //Utils1();
  //AltCompares1();
  //Mutators1();
  //AltCompares2();
  //misc();
}

void Mutators1() {
  void print(List!(int) list) {
    foreach(int i; list) {
      Stdout.put(i).cr();
    }
  }
  MutableList!(int) list = new ArrayList!(int)();
  list.append(1).append(2).append(3).append(4);
  print(list);
  list.mutate!(Reverse)();
  print(list);
}

void misc() {
  struct S {
    int i;
  }

  ConcurrentHashMap!(int,int) mapII;  //This is the only one that works.
  //ConcurrentHashMap!(int,char[]) mapIC;
  //ConcurrentHashMap!(char[],int) mapCI;
  //ConcurrentHashMap!(char[],char[]) mapCC;

  //ConcurrentHashMap!(int,S) mapIS;
  //ConcurrentHashMap!(S,int) mapSI;
  //ConcurrentHashMap!(S,S) mapSS;

  //ConcurrentHashMap!(float,float) mapFF;
}

/*************************
  For AltCompare tests
 *************************/
class Test {
  int A;
  int B;
  this (int A, int B) {
    this.A = A;
    this.B = B;
  }
  int opEquals(Object o) {
    Test t = cast(Test)o;
    assert(t);
    return t.A == A;
  }
  uint toHash() {
    return cast(uint) A;
  }
}

int myEquals(Test a, Test b) {
  return a.B == b.B;
}
int myCmp(Test a, Test b) {
  return a.B - b.B;
}
uint myHash(Test t) {
  return cast(uint)t.B;
}

void AltCompares1() {
  alias HashMap!(Test, char[], myEquals, myCmp, myHash) MyMap;
  MutableMap!(Test,char[]) map = new MyMap(5);
  map[new Test(5,6)] = "six, not five";
  map[new Test(10,9)] = "nine, not ten";
  Stdout.put(map[new Test(7,6)]).cr();
  Stdout.put(map[new Test(15,9)]).cr();
}

void AltCompares2() {
  /+alias ConcurrentHashMap!(Test, char[], myEquals, myCmp, myHash) MyMap;
  MutableMap!(Test,char[]) map = new MyMap(5);
  map[new Test(5,6)] = "six, not five";
  map[new Test(10,9)] = "nine, not ten";
  Stdout.put(map[new Test(7,6)]).cr();
  Stdout.put(map[new Test(15,9)]).cr();+/
}
/**************************
  End AltCompares
 **************************/


void Utils1() {
  void print(uint hash) {
    Stdout.put(hash).cr();
  }

  struct S {
    int i;
    float f;
    uint toHash() {
      return 0;
    }
  }
  static S s = { 5 ,7.5 };
  print(Util!(S).hash(s));
  print(Util!(S*).hash(&s));

  print(Util!(float).hash(85.78));
  print(Util!(char[]).hash("foobar"));
  print(Util!(long).hash(987616513l));
}

void ConcurrentHashMap1() {
  ConcurrentHashMap!(int,int) mapA = new ConcurrentHashMap!(int,int)();
  mapA.put(1,2);
  mapA.put(3,6);
  mapA.put(4,8);

  assert(mapA.get(4) == 8);
  mapA[5] = 10;
  mapA.remove(5);
  assert(mapA.keySet().contains(3));
  assert(mapA.valueSet().contains(8));
  assert(mapA.size == 3);

  int value;
  mapA.remove(4,value);
  mapA.put(5,value);
  assert(mapA[5] == value);
}

void HashMap1() {
  MutableMap!(char[], int) mapA = new HashMap!(char[],int)();
  mapA["foo"] = 3;
  mapA["bar"] = 5;
  mapA["foobar"] = 6;

  assert(mapA["bar"] == 5);
  
  MutableMapIterator!(char[], int) iter = mapA.iterator();
  while (iter.hasNext()) {
    char[] s;
    int i;
    iter.next(s,i);
    Stdout.put(s).put(": ").put(i).cr();
  }
}

void HashSet1() {
  MutableSet!(int) setA = new HashSet!(int)();
  setA.add(5);
  setA.add(7);
  setA.add(6);
  setA.add(13);
  MutableIterator!(int) iter = setA.iterator();
  while(iter.hasNext()) {
    int i = iter.next();
    if (i%2==0) {
      iter.remove();
      Stdout.put("No evens allowed!!! (Removed number ").put(i).put(")").cr();
    } else {
      Stdout.put(i).cr();
    }
  }
}

//An example of how to populate and use an ImmutableList
void ImmutableList1() {
  int count;
  int max = 10;
  
  bit addElements(out int i) {
    i = count++;
    return cast(bit)(i<max);
  }

  void print(List!(int) list) {
    Stdout.put("Content:").cr();
    Iterator!(int) iter = list.iterator();
    while (iter.hasNext()) {
      Stdout.put(iter.next()).cr();
    }
    Stdout.put("--------").cr();
  }


  ImmutableList!(int) list1 = new ImmutableList!(int)(&addElements);
  print(list1);
  count -= 5;
  max = 15;
  ImmutableList!(int) list2 = new ImmutableList!(int)(&addElements);
  print(list2);
  ImmutableList!(int) common = list1.intersection(list2);
  print(common);
}
