module mango.test.xml;

import  mango.io.Uri,
        mango.io.Stdout,
        mango.io.Token,
        mango.io.Reader,
        mango.io.Writer,
        mango.io.Buffer,
        mango.io.Socket,
        mango.io.Conduit,
        mango.io.FilePath,
        mango.io.FileStyle,
        mango.io.FileProxy,
        mango.io.Tokenizer,
        mango.io.TextReader,
        mango.io.TextWriter,
        mango.io.FileBucket,
        mango.io.FileConduit,
        mango.io.TextLayout,
        mango.io.TextFormat,
        mango.io.ColumnWriter,
        mango.io.DisplayWriter,
        mango.io.PickleReader,
        mango.io.PickleWriter,
        mango.io.SocketConduit,
        mango.io.SocketListener,
        mango.io.ArrayAllocator,
        mango.io.PickleRegistry,
        mango.io.DatagramSocket,
        mango.io.MulticastSocket;

        // need these for reading & writing classes
import  mango.io.model.IReader,
        mango.io.model.IWriter,
        mango.io.model.IPickle;

import mango.log.Logger;
import mango.log.ConsoleAppender;

import  mango.icu.UMango,
        mango.icu.UString,
        mango.icu.UDomainName,
        mango.icu.UStringPrep,
        mango.icu.UConverter,
        mango.icu.USet,
        mango.icu.UCollator,
        mango.icu.UTimeZone,
        mango.icu.UEnumeration,
        mango.icu.UMessageFormat;

import	mango.xml.dom.all,
  mango.xml.dom.parser.teqXML,
  mango.xml.rpc.client,
  mango.xml.rpc.objects,
  mango.xml.rpc.server;

import mango.servlet.ServletProvider;
import mango.http.server.HttpServer;


/*******************************************************************************


*******************************************************************************/


void testRPCClient ()
{
  XmlRpcClient client = new XmlRpcClient("http://scripts.incutio.com/xmlrpc/simpleserver.php");
  MethodCall mc = new MethodCall(new UString("test.getTime"));
  MethodResponse mr = client.sendMethodCall(mc);
  Stdout.put("Time: ").put(mr.getParams()[0].getString()).cr();
}


/*******************************************************************************


*******************************************************************************/


void testRPCServer ()
{	
  MethodResponse testMethod(MethodCall mc) {
    MethodResponse mr = new MethodResponse();
    Value val = new Value(new UString("means nothing"));
    mr.appendParam(val);
    return mr;
  }
  Logger logger = Logger.getLogger ("neuralNexus");
  logger.setLevel(Logger.Level.Trace);
  logger.addAppender(new ConsoleAppender());
  
  //The local port to bind to
  InternetAddress bindTo = new InternetAddress(8080);
  
  //The ServletProvider
  ServletProvider sp = new ServletProvider();
  
  //This is the nnServer in servlet form
  XmlRpcServlet s = new XmlRpcServlet(logger);
  s.addCallHandler(new UString("test.getTime"), &testMethod);
  IRegisteredServlet irs = sp.addServlet (s, "XmlRpc");
  sp.addMapping ("/RPC2", irs);
  
  //The HTTP Server
  HttpServer httpServer = new HttpServer(sp, bindTo, 1, logger);
  
  //Go!
  httpServer.start();

  XmlRpcClient client = new XmlRpcClient("http://localhost:8080/RPC2");
  MethodCall mc = new MethodCall(new UString("test.getTime"));
  MethodResponse mr = client.sendMethodCall(mc);
  Stdout.put("Time: ").put(mr.getParams()[0].getString()).cr();

  delete httpServer;
}


/*******************************************************************************


*******************************************************************************/

void testDOM ()
{
	Document doc = new DocumentImpl(null);
	Element root = doc.createElement(new UString("root"));
	Element text = doc.createElement(new UString("text"));
	root.appendChild(text);
	CDATASection data = doc.createCDATASection(new UString("this is my text"));
	text.appendChild(data);
	Element secText = doc.createElement(new UString("text"));
	assert (secText !== null);
	root.appendChild(secText);
	assert (root.childNodes.length == 2);
	assert (root.childNodes.item(0) === text);
	assert (root.childNodes.item(1) === secText);
	assert (cast(Element)secText.parentNode === root);
	assert (cast(Element)secText.previousSibling !== null);
	assert (cast(Element)text.nextSibling === secText);

	Element top = doc.createElement(new UString("top"));
	root.insertBefore(top, text);
	assert (root.childNodes.length == 3);
	assert (root.childNodes.item(0) === top);
	assert (root.childNodes.item(1) === text);
	assert (root.childNodes.item(2) === secText);
	assert (text.previousSibling === top);
}

/*******************************************************************************


*******************************************************************************/

char[] xmlString = "<?xml version=\"1.0\" encoding=\"utf-8\"?>
<!--A collection of recipies-->
<collection>
  <description>
     Some recipes used for the XML tutorial.
  </description>
  <recipe>
    <![CDATA[<title>Beef Parmesan with Garlic Angel Hair Pasta</title>]]>
    <ingredient name=\"beef cube steak\" amount=\"1.5\" unit=\"pound\"/>
    <preparation>
      <step>
        Preheat oven to 350 degrees F (175 degrees C).
      </step>
    </preparation>
    <comment>
      Make the meat ahead of &quot;time&quot;, &amp; refrigerate over night, the acid in the
      tomato sauce will tenderize the meat even more. If you do this, save the
      mozzarella till the last minute.
    </comment>
    <nutrition calories='11\"67' fat=\"23\" carbohydrates=\"45\" protein=\"32\"/>
  </recipe>
</collection>";

void testTeqParser (char[] file=null, char[] outfile=null)
{	
  IBuffer buff;
  if (file is null) {
	buff = new GrowableBuffer(xmlString.length);
	buff.setContent(xmlString, xmlString.length);
  } else {
    FileConduit fc = new FileConduit(file);
    buff = fc.createBuffer();
  }
  Document doc = TeqXML.Parse(buff);
  doc.dump(Stdout, 0);
  Stdout.cr();
  Stdout.setEncoder (new StringEncoder16 ("utf8"));
  Stdout.setEncoder (new StringEncoder32 ("utf8"));
  if (outfile) {
    FileConduit fc = new FileConduit(outfile, FileStyle.ReadWriteCreate);
    TeqXML.Write(doc, fc.createBuffer(), "utf8");
    TeqXML.Write(doc, Stdout.getBuffer(), "utf8");
  } else {
    TeqXML.Write(doc, Stdout.getBuffer(), "utf8");
  }
  Stdout.cr().flush();
}

void main(char[][] args) {
  try {
    testRPCServer();
    testRPCClient();
    testDOM();
    if (args.length == 3)
      testTeqParser(args[1], args[2]);
    else if (args.length == 2)
      testTeqParser(args[1]);
    else
      testTeqParser();
  } catch (Object o) {
    printf(o.toString() ~ "\n\0");
  }
}
