/*******************************************************************************

        @file Text.d
        
        Copyright (c) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.

        4. Derivative works are permitted, but they must carry this notice
           in full and credit the original source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        
        @version        Initial version, April 2004      
        @author         Kris


*******************************************************************************/

module mango.utils.Text;

version (Ares)
         private import  std.c.ctype;
      else
         private import  std.ctype;


/******************************************************************************

******************************************************************************/

extern (C)
{
        char* memchr (char *, char, uint);
        int   memcmp (char *, char *, uint);
}       

/******************************************************************************

        Placeholder for a variety of wee functions. Some of these are
        handy for Java programmers, but the primary reason for their
        existance is that they don't allocate memory (unlike Phobos).
        That is, processing is performed in-place or within a client-
        provided temporary buffer.

******************************************************************************/

abstract class Text
{
        /**********************************************************************

                Replace all instances of one char with another (in place)

        **********************************************************************/

        final static char[] replace (char[] source, char match, char replacement)
        {
                char *p;
                char *scan = &source[0];
                int length = source.length;

                while ((p = memchr (scan, match, length)) != null)
                      {
                      *p = replacement;
                      length -= (p - scan);
                      scan = p;
                      }
                return source;
        }

        /**********************************************************************

                Return the index of the first instance of 'match'

        **********************************************************************/

        final static int indexOf (char[] source, char match)
        {
                return indexOf (source, match, 0);
        }

        /**********************************************************************

                Return the index of the first instance of 'match', starting
                at position 'start'
                
        **********************************************************************/

        final static int indexOf (char[] source, char match, int start)
        {
                if (start < source.length)
                   {
                   char *p = memchr (&source[start], match, source.length - start);
                   if (p)
                       return start + (p - &source[start]);
                   }
                return -1;
        }

        /**********************************************************************

                Return the index of the first instance of 'match'

        **********************************************************************/

        final static int indexOf (char[] source, char[] match)
        {
                return indexOf (source, match, 0);
        }

        /**********************************************************************

                Return the index of the first instance of 'match', starting
                at position 'start'
                
        **********************************************************************/

        final static int indexOf (char[] source, char[] match, int start)
        {
                int length = match.length;
                int extent = source.length - length + 1;
                
                for (; start < extent; ++start)
                    {
                    start = indexOf (source, match[0], start);
                    if (start < 0)
                        break;
                    else
                       if (memcmp (&source[start], match, length) == 0)
                           return start;
                    }
                return -1;
        }

        /**********************************************************************

                Trim the provided string by stripping whitespace from 
                both ends. Returns a slice of the original content.

        **********************************************************************/

        final static char[] trim (char[] source)
        {
                if (source.length)
                   {
                   int  front,
                        back = source.length;

                   while (front < back && isspace(source[front]))
                          ++front;

                   while (back > front && isspace(source[back-1]))
                          --back;
                   
                   if (front > 0 || back < source.length)   
                       return source[front..back];
                   } 
                return source;
        }

        /**********************************************************************

                
        **********************************************************************/

        final static char[][] split (char[] src, char[] delim)
        {
                int             pos,
                                mark;
                char[][]        ret;

                assert (delim.length);
                while ((pos = indexOf (src, delim, pos)) >= 0)
                      { 
                      ret ~= src [mark..pos];
                      pos += delim.length;
                      mark = pos;
                      }

                if (mark < src.length)
                    ret ~= src [mark..src.length];
                return ret;                                      
        }

        /**********************************************************************

                in-place conversion to lowercase 

        **********************************************************************/

        final static char[] tolower (inout char[] src)
        {
                foreach (int i, char c; src)
                         if (c >= 'A' && c <= 'Z')
                             src[i] = c + ('a' - 'A');
                return src;
        }
}

